"use strict";

const {
  spawnSync,
  execSync
} = require("child_process");
const {
  executeMethodsThatMayThrowException
} = require("../General/wrapperFunctionToExecuteAFunction");
const {
  executeSynchronizedCommands
} = require("../General/executeSyncCommands");

/**
 * @function getGitUserName - to get the git username
 * @returns {string | null} - returns the username or null if command execution encounters an exception
 */
function getGitUserName() {
  const userHandler = () => {
    const userName = exeSpawnSync("git", ["config", "user.email"]);
    if (userName == null) {
      return null;
    }
    return userName.pop().split("@")[0];
  };
  return executeMethodsThatMayThrowException(null, 'kindly initialize git properly', userHandler, null);
}

/**
 * @function getRepoName - to get the git repository name
 * @returns {string | null} - returns the git reponame or null if command execution encounters an exception
 */
function getRepoName() {
  const repoHandler = () => {
    const repoName = exeSpawnSync("git", ["remote", "get-url", "origin"]);
    if (repoName == null) {
      return null;
    }
    return repoName.pop().split("/").pop().split(".")[0];
  };
  return executeMethodsThatMayThrowException(null, 'kindly initialize git properly', repoHandler, null);
}

/**
 * @function getBranchName - to get the current branch name
 * @returns {string | null} - returns the git reponame or null if command execution encounters an exception
 */
function getBranchName() {
  const branchHandler = () => {
    var branchName = exeSpawnSync("git", ["rev-parse", "--abbrev-ref", "HEAD"]); // to get current branch
    if (branchName == null) {
      return null;
    }
    return branchName.shift().trim();
  };
  return executeMethodsThatMayThrowException(null, 'kindly initialize git properly', branchHandler, null);
}

/**
 * 
 * @param {Function} command - comamnd/function to be executed
 * @param {*} params - parameters to be passed to the command/function to be executed
 * @returns {null | any}
 */
function exeSpawnSync(command, params = []) {
  const demon_process = spawnSync(command, params, {
    shell: true
  });
  if (!(demon_process.status == 0)) {
    return null;
  }
  return demon_process.stdout.toString().split("\n").filter(Boolean);
}

/**
 * @function isGitInitialized - to check if git is initialized
 * @returns {boolean} - inidcating whether the repo is git initialized or not
 */
function isGitInitialized() {
  const isGit = executeSynchronizedCommands(execSync, ['git status'], '', "Kindly initialize git using \"git init\" ", false, true);
  return isGit;
}
module.exports = {
  getBranchName,
  getGitUserName,
  getRepoName,
  isGitInitialized
};