"use strict";

/* eslint-disable no-console */
const {
  execSync
} = require('child_process');
const {
  readdirSync
} = require('fs');
const path = require('path');
const initializeHusky = require('./initializeHusky');
const configurePrecommitHook = require('./configurePrecommitHook');
const {
  getNodeModulesPath
} = require('../General/getNodeModulesPath');
const {
  executeSynchronizedCommands
} = require('../General/executeSyncCommands');
const {
  Logger
} = require('../Logger/Logger');
let isCustomPrecommitConfigurationSuccessful = false;

/**
 * @function setupHusky - to setup custom precommit hook using husky plugin
 * @returns {boolean} - indicating whether entire custom precommit hook setup using husky process is success or failure
 */
function setupHusky() {
  let isHuskyPackageInstalled = installHuskyPackage();
  if (isHuskyPackageInstalled) {
    let isHuskyInitializedSuccessfully = initializeHusky();
    if (isHuskyInitializedSuccessfully) {
      Logger.log(Logger.SUCCESS_TYPE, 'Husky installation successful');
      isCustomPrecommitConfigurationSuccessful = configurePrecommitHook();
      if (isCustomPrecommitConfigurationSuccessful) {
        Logger.log(Logger.SUCCESS_TYPE, 'Custom pre commit hook setup was successful');
        return true;
      } else {
        Logger.log(Logger.FAILURE_TYPE, 'Couldn\'t configure custom pre commit hook');
        return false;
      }
    } else {
      return false;
    }
  } else {
    Logger.log(Logger.FAILURE_TYPE, 'Some issue in installing husky package.');
    return false;
  }
}

/**
 * @function installHuskyPackage - installs husky package if not installed for the project
 * @returns {boolean} - indicating if husky package installed successfully or not
 */
function installHuskyPackage() {
  const nodeModulesPathOfProject = path.join(getNodeModulesPath(), 'node_modules');
  let pluginsInNodeModules = executeSynchronizedCommands(readdirSync, [nodeModulesPathOfProject], '', 'Unable to get directories in node_modules when checking if husky is installed', true, false);
  let husky = {
    packageName: 'husky',
    version: '^7.0.4'
  };
  let isHuskyInstalled = pluginsInNodeModules.includes(husky.packageName) ? true : false;
  if (!isHuskyInstalled) {
    Logger.log(Logger.INFO_TYPE, 'Installing husky package ....');
    let installCommand = `npm install ${husky.packageName}@${husky.version} --save-dev`;
    let isHuskyPackageInstalled = executeSynchronizedCommands(execSync, [installCommand, {
      stdio: 'inherit'
    }], 'Husky package Installed Successfully', 'Unable to install husky package', false, true);
    return isHuskyPackageInstalled;
  }
  Logger.log(Logger.SUCCESS_TYPE, 'Husky package already installed!');
  return true;
}
module.exports = {
  setupHusky
};